/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "multiaccessory.h"

#define MULTI_ACCESSORY1_ASPID "/sample/multiplicity1"
#define MULTI_ACCESSORY2_ASPID "/sample/multiplicity2"
#define MULTI_ACCESSORY1_CHANNELID 1000
#define MULTI_ACCESSORY2_CHANNELID 2000

struct priv {
	sap_agent_h agent;
	sap_socket_h socket;
	sap_peer_agent_h peer_agent;
	int agent_no;
};

static struct priv priv_agent1 = { 0 }, priv_agent2 = { 0 };

static void on_data_recieved(sap_socket_h socket,
			     unsigned short int channel_id,
			     unsigned int payload_length,
			     void *buffer,
			     void *user_data)
{
	struct priv *priv = NULL;
	priv = (struct priv *)user_data;

	dlog_print(DLOG_DEBUG, TAG, "received data: %s, len:%d", buffer, payload_length);
	update_ui(buffer);
}

static void on_service_connection_terminated(sap_peer_agent_h peer_agent,
					     sap_socket_h socket,
					     sap_service_connection_terminated_reason_e result,
					     void *user_data)
{
	struct priv *priv = NULL;
	priv = (struct priv *)user_data;
	switch (result) {
	case SAP_CONNECTION_TERMINATED_REASON_PEER_DISCONNECTED:
		dlog_print(DLOG_INFO, TAG, "connection got terminated");
		if (priv->agent_no == 1) {
			sap_socket_destroy(priv_agent1.socket);
			priv_agent1.socket = NULL;
			update_ui("Peer Disconnected for /sample/multi1");
		} else if (priv->agent_no == 2) {
			sap_socket_destroy(priv_agent2.socket);
			priv_agent2.socket = NULL;
			update_ui("Peer Disconnected for /sample/multi2");
		}
		break;

	case SAP_CONNECTION_TERMINATED_REASON_DEVICE_DETACHED:
		dlog_print(DLOG_INFO, TAG, "Disconnected Device Detached");
		update_ui("Disconnected Device Detached");
		break;

	case SAP_CONNECTION_TERMINATED_REASON_UNKNOWN:
		update_ui("Disconnected Unknown Reason");
		break;
	}

}
static void on_service_connection_created(sap_peer_agent_h peer_agent,
					  sap_socket_h socket,
					  sap_service_connection_result_e result,
					  void *user_data)
{
	struct priv *priv = NULL;
	priv = (struct priv *)user_data;

	switch (result) {

	case SAP_CONNECTION_SUCCESS:
		if (priv->agent_no == 1) {
			sap_peer_agent_set_service_connection_terminated_cb(priv_agent1.peer_agent,
									    on_service_connection_terminated,
									    priv);

			sap_socket_set_data_received_cb(socket, on_data_recieved, priv);
			priv_agent1.socket = socket;
			update_ui("Connection Established for /sample/multi1");
		} else if (priv->agent_no == 2) {
			sap_peer_agent_set_service_connection_terminated_cb(priv_agent2.peer_agent,
									    on_service_connection_terminated,
									    priv);

			sap_socket_set_data_received_cb(socket, on_data_recieved, priv);
			priv_agent2.socket = socket;
			update_ui("Connection Established for /sample/multi2");
		}
		break;

	case SAP_CONNECTION_ALREADY_EXIST:

		if (priv->agent_no == 1) {
			priv_agent1.socket = socket;
		} else if (priv->agent_no == 2) {
			priv_agent2.socket = socket;
		}
		update_ui("Multi AccessoryConnection Already Exist");
		break;

	case SAP_CONNECTION_FAILURE_DEVICE_UNREACHABLE:
		dlog_print(DLOG_DEBUG, TAG, "device is not unreachable");
		break;

	case SAP_CONNECTION_FAILURE_INVALID_PEERAGENT:
		dlog_print(DLOG_DEBUG, TAG, "invalid peer agent");
		break;

	case SAP_CONNECTION_FAILURE_NETWORK:
		dlog_print(DLOG_DEBUG, TAG, "network failure");
		break;

	case SAP_CONNECTION_FAILURE_PEERAGENT_NO_RESPONSE:
		dlog_print(DLOG_DEBUG, TAG, "peer agent is no response");
		break;

	case SAP_CONNECTION_FAILURE_PEERAGENT_REJECTED:
		dlog_print(DLOG_DEBUG, TAG, "peer agent is rejected");
		break;

	case SAP_CONNECTION_IN_PROGRESS:
		dlog_print(DLOG_DEBUG, TAG, "connection is in progress");
		break;

	case SAP_CONNECTION_PEER_AGENT_NOT_SUPPORTED:
		dlog_print(DLOG_ERROR, TAG, "peer agent not supported");
		break;

	case SAP_CONNECTION_FAILURE_UNKNOWN:
		dlog_print(DLOG_DEBUG, TAG, "unknown error");
		break;
	}
}
static gboolean _create_service_connection(gpointer user_data)
{
	struct priv *priv = NULL;
	sap_result_e result = SAP_RESULT_FAILURE;

	priv = (struct priv *)user_data;

	result = sap_agent_request_service_connection(priv->agent,
						      priv->peer_agent,
						      on_service_connection_created,
						      priv);

	if (result == SAP_RESULT_SUCCESS) {
		dlog_print(DLOG_INFO, TAG, "req service conn call succeeded");
	} else {
		dlog_print(DLOG_ERROR, TAG, "req service conn call is failed (%d)", result);
	}
	return FALSE;
}

GLboolean request_service_connection(int i)
{
	if (i == 1) {
		g_idle_add(_create_service_connection, &priv_agent1);
	} else if (i == 2) {
		g_idle_add(_create_service_connection, &priv_agent2);
	}
	return TRUE;
}

void on_peer_agent_updated(sap_peer_agent_h peer_agent,
			   sap_peer_agent_status_e peer_status,
			   sap_peer_agent_found_result_e result,
			   void *user_data)
{
	struct priv *priv = NULL;
	priv = (struct priv *)user_data;
	dlog_print(DLOG_DEBUG, TAG, "%s", __func__);
	switch (result) {

	case SAP_PEER_AGENT_FOUND_RESULT_DEVICE_NOT_CONNECTED:
		dlog_print(DLOG_ERROR, TAG, "device is not connected");
		break;

	case SAP_PEER_AGENT_FOUND_RESULT_FOUND:

		if (peer_status == SAP_PEER_AGENT_STATUS_AVAILABLE) {
			if (priv->agent_no == 1) {
				priv_agent1.peer_agent = peer_agent;
				request_service_connection(1);
			} else if (priv->agent_no == 2) {
				priv_agent2.peer_agent = peer_agent;
				request_service_connection(2);
			}
		} else {
			dlog_print(DLOG_DEBUG, TAG, "peer agent removed");
			sap_peer_agent_destroy(peer_agent);
		}
		break;

	case SAP_PEER_AGENT_FOUND_RESULT_SERVICE_NOT_FOUND:
		dlog_print(DLOG_ERROR, TAG, "service not found");
		break;

	case SAP_PEER_AGENT_FOUND_RESULT_TIMEDOUT:
		dlog_print(DLOG_ERROR, TAG, "peer agent find timed out");
		break;

	case SAP_PEER_AGENT_FOUND_RESULT_INTERNAL_ERROR:
		dlog_print(DLOG_ERROR, TAG, "peer agent find search failed");
		break;

	default:
		dlog_print(DLOG_ERROR, TAG, "unknown status (%d)", peer_status);
		break;

	}
}

static gboolean _terminate_service_connection(gpointer user_data)
{
	struct priv *priv = NULL;
	sap_result_e result = SAP_RESULT_FAILURE;

	priv = (struct priv *)user_data;

	result = sap_peer_agent_terminate_service_connection(priv->peer_agent);

	if (result == SAP_RESULT_SUCCESS) {
		update_ui("Connection Terminated");
		sap_socket_destroy(priv->socket);
		priv->socket = NULL;
		dlog_print(DLOG_INFO, TAG, "priv:%u", priv);
	} else {
		update_ui("Connection Termination Failed");
	}

	return FALSE;
}

GLboolean terminate_service_connection(int i)
{
	if (i == 1) {
		g_idle_add(_terminate_service_connection, &priv_agent1);
	} else if (i == 2) {
		g_idle_add(_terminate_service_connection, &priv_agent2);
	}
	return TRUE;
}

static gboolean _find_peer_agent(gpointer user_data)
{
	struct priv *priv = NULL;
	sap_result_e result = SAP_RESULT_FAILURE;

	priv = (struct priv *)user_data;

	result = sap_agent_find_peer_agent(priv->agent, on_peer_agent_updated, priv);

	if (result == SAP_RESULT_SUCCESS) {
		dlog_print(DLOG_DEBUG, TAG, "find peer call succeeded %d", result);
	} else {
		dlog_print(DLOG_ERROR, TAG, "findsap_peer_agent_s is failed (%d)", result);
	}
	return FALSE;
}

GLboolean find_peers(int i)
{
	if (i == 1) {
		g_idle_add(_find_peer_agent, &priv_agent1);
	} else if (i == 2) {
		g_idle_add(_find_peer_agent, &priv_agent2);
	}
	return TRUE;

}
GLboolean send_data(int i, char *message)
{
	int result;
	if (i == 1) {
		if (priv_agent1.socket) {
			result = sap_socket_send_data(priv_agent1.socket, MULTI_ACCESSORY1_CHANNELID, strlen(message), message);
		}
	} else if (i == 2) {
		if (priv_agent2.socket) {
			result = sap_socket_send_data(priv_agent2.socket, MULTI_ACCESSORY2_CHANNELID, strlen(message), message);
		}
	}
	return TRUE;

}

static void on_agent_initialized(sap_agent_h agent,
				 sap_agent_initialized_result_e result,
				 void *user_data)
{
	struct priv *priv = NULL;
	priv = (struct priv *)user_data;

	switch (result) {
	case SAP_AGENT_INITIALIZED_RESULT_SUCCESS:

		if (priv->agent_no == 1) {
			priv_agent1.agent = agent;
			g_idle_add(_find_peer_agent, &priv_agent1);
		} else if (priv->agent_no == 2) {
			priv_agent2.agent = agent;
			g_idle_add(_find_peer_agent, &priv_agent2);
		}

		break;

	case SAP_AGENT_INITIALIZED_RESULT_DUPLICATED:
		dlog_print(DLOG_ERROR, TAG, "duplicate registration");
		break;

	case SAP_AGENT_INITIALIZED_RESULT_INVALID_ARGUMENTS:
		dlog_print(DLOG_ERROR, TAG, "invalid arguments");
		break;

	case SAP_AGENT_INITIALIZED_RESULT_INTERNAL_ERROR:
		dlog_print(DLOG_ERROR, TAG, "internal sap error");

		break;

	default:
		dlog_print(DLOG_ERROR, TAG, "unknown status (%d)", result);
		break;
	}
}

static void on_device_status_changed(sap_device_status_e status, sap_transport_type_e transport_type,
				     void *user_data)
{
	switch (transport_type) {
	case SAP_TRANSPORT_TYPE_BT:
		dlog_print(DLOG_DEBUG, TAG, "connectivity type(%d): bt", transport_type);
		break;

	case SAP_TRANSPORT_TYPE_BLE:
		dlog_print(DLOG_DEBUG, TAG, "connectivity type(%d): ble", transport_type);
		break;

	case SAP_TRANSPORT_TYPE_TCP:
		dlog_print(DLOG_DEBUG, TAG, "connectivity type(%d): tcp/ip", transport_type);
		break;

	case SAP_TRANSPORT_TYPE_USB:
		dlog_print(DLOG_DEBUG, TAG, "connectivity type(%d): usb", transport_type);
		break;

	case SAP_TRANSPORT_TYPE_MOBILE:
		dlog_print(DLOG_DEBUG, TAG, "connectivity type(%d): mobile", transport_type);
		break;

	default:
		dlog_print(DLOG_ERROR, TAG, "unknown connectivity type (%d)", transport_type);
		break;
	}

	switch (status) {
	case SAP_DEVICE_STATUS_DETACHED:
		break;

	case SAP_DEVICE_STATUS_ATTACHED:
		break;

	default:
		dlog_print(DLOG_ERROR, TAG, "unknown status (%d)", status);
		break;
	}

}

gboolean agent_initialize()
{
	int result_agent1 = 0, result_agent2 = 0;

	do {
		result_agent1 = sap_agent_initialize(priv_agent1.agent, MULTI_ACCESSORY1_ASPID, SAP_AGENT_ROLE_CONSUMER,
						     on_agent_initialized, &priv_agent1);

		result_agent2 = sap_agent_initialize(priv_agent2.agent, MULTI_ACCESSORY2_ASPID, SAP_AGENT_ROLE_CONSUMER,
						     on_agent_initialized, &priv_agent2);
		dlog_print(DLOG_INFO, TAG, "getRegisteredServiceAgent() >>> %d", result_agent1);
	} while (result_agent1 != SAP_RESULT_SUCCESS || result_agent2 != SAP_RESULT_SUCCESS);

	return TRUE;
}

void initialize_sap()
{
	dlog_print(DLOG_ERROR, TAG, "creating agents");
	sap_agent_h agent1 = NULL;
	sap_agent_h agent2 = NULL;

	sap_agent_create(&agent1);
	sap_agent_create(&agent2);

	if (agent1 == NULL)
		dlog_print(DLOG_ERROR, TAG, "ERROR in creating agent");
	else
		dlog_print(DLOG_INFO, TAG, "SUCCESSFULLY create sap agent");

	if (agent2 == NULL)
		dlog_print(DLOG_ERROR, TAG, "ERROR in creating agent");
	else
		dlog_print(DLOG_INFO, TAG, "SUCCESSFULLY create sap agent");

	priv_agent1.agent = agent1;
	priv_agent2.agent = agent2;

	priv_agent1.agent_no = 1;
	priv_agent2.agent_no = 2;


	sap_set_device_status_changed_cb(on_device_status_changed, NULL);

	agent_initialize();
}
