/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <sap.h>
#include "multiprovider.h"

#define MULTI_ACCESSORY1_ASPID "/sample/multiplicity1"
#define MULTI_ACCESSORY2_ASPID "/sample/multiplicity2"
#define MULTI_ACCESSORY1_CHANNELID 1000
#define MULTI_ACCESSORY2_CHANNELID 2000

struct priv {
	sap_agent_h agent;
	sap_socket_h socket;
	sap_peer_agent_h peer_agent;
	int agent_no;
};

static struct priv priv_agent1 = { 0 }, priv_agent2 = { 0 };

static void on_data_recieved(sap_socket_h socket,
			     unsigned short int channel_id,
			     unsigned int payload_length,
			     void *buffer,
			     void *user_data)
{
	struct priv *priv = NULL;
	priv = (struct priv *)user_data;
	time_t raw_time;
	struct tm* time_info;
	char timestamp[30];

	dlog_print(DLOG_INFO, TAG, "received data: %s, length:%d ", buffer, payload_length);

	time(&raw_time);
	time_info = localtime(&raw_time);
	sprintf (timestamp, "Time: %d:%s%d:%d", time_info->tm_hour, time_info->tm_min<10? "0" : "", time_info->tm_min,time_info->tm_sec);

	char *msg = g_strdup_printf("%s%s", (char *)buffer,timestamp);
	update_ui(msg);

	if (priv->agent_no == 1) {
		send_data(1, msg);
	} else if (priv->agent_no == 2) {
		send_data(2, msg);
	}
	g_free(msg);
}

static void on_service_connection_terminated(sap_peer_agent_h peer_agent,
					     sap_socket_h socket,
					     sap_service_connection_terminated_reason_e result,
					     void *user_data)
{
	struct priv *priv = NULL;
	priv = (struct priv *)user_data;
	switch (result) {
	case SAP_CONNECTION_TERMINATED_REASON_PEER_DISCONNECTED:
		if (priv->agent_no == 1) {
			update_ui("Peer Disconnected for /sample/multi1");
		} else if (priv->agent_no == 2) {
			update_ui("Peer Disconnected for /sample/multi2");
		}
		break;

	case SAP_CONNECTION_TERMINATED_REASON_DEVICE_DETACHED:
		update_ui("Disconnected Device Detached");
		break;

	case SAP_CONNECTION_TERMINATED_REASON_UNKNOWN:
		update_ui("Disconnected Unknown Reason");
		break;
	}

	if (priv->agent_no == 1) {
		sap_socket_destroy(priv_agent1.socket);
		priv_agent1.socket = NULL;
	} else if (priv->agent_no == 2) {
		sap_socket_destroy(priv_agent2.socket);
		priv_agent2.socket = NULL;
	}
	dlog_print(DLOG_INFO, TAG, "status:%d", result);
}

void on_peer_agent_updated(sap_peer_agent_h peer_agent,
			   sap_peer_agent_status_e peer_status,
			   sap_peer_agent_found_result_e result,
			   void *user_data)
{
	struct priv *priv = NULL;
	priv = (struct priv *)user_data;
	switch (result) {

	case SAP_PEER_AGENT_FOUND_RESULT_DEVICE_NOT_CONNECTED:
		dlog_print(DLOG_ERROR, TAG, "device is not connected");
		break;

	case SAP_PEER_AGENT_FOUND_RESULT_FOUND:

		if (peer_status == SAP_PEER_AGENT_STATUS_AVAILABLE) {
			dlog_print(DLOG_INFO, TAG, " peer status %d for agent %d", peer_status, priv->agent_no);
			if (priv->agent_no == 1) {
				priv_agent1.peer_agent = peer_agent;
				update_ui("Peer Agent Found : /sample/multi1");
			} else if (priv->agent_no == 2) {
				priv_agent2.peer_agent = peer_agent;
				update_ui("Peer Agent Found : /sample/multi2");
			}
		} else {
			dlog_print(DLOG_DEBUG, TAG, "peer agent removed");
			sap_peer_agent_destroy(peer_agent);
		}
		break;

	case SAP_PEER_AGENT_FOUND_RESULT_SERVICE_NOT_FOUND:
		dlog_print(DLOG_ERROR, TAG, "service not found");
		break;

	case SAP_PEER_AGENT_FOUND_RESULT_TIMEDOUT:
		dlog_print(DLOG_ERROR, TAG, "peer agent find timed out");
		break;

	case SAP_PEER_AGENT_FOUND_RESULT_INTERNAL_ERROR:
		dlog_print(DLOG_ERROR, TAG, "peer agent find search failed");
		break;

	default:
		dlog_print(DLOG_ERROR, TAG, "unknown status (%d)", peer_status);
		break;

	}
}

static gboolean _find_peer_agent(gpointer user_data)
{
	struct priv *priv = NULL;
	sap_result_e result = SAP_RESULT_FAILURE;

	dlog_print(DLOG_DEBUG, TAG, "%s", __func__);
	priv = (struct priv *)user_data;

	result = sap_agent_find_peer_agent(priv->agent, on_peer_agent_updated, priv);

	if (result == SAP_RESULT_SUCCESS) {
		dlog_print(DLOG_DEBUG, TAG, "find peer call succeeded %d", result);
	} else {
		dlog_print(DLOG_ERROR, TAG, "findsap_peer_agent_s is failed (%d)", result);
	}

	return FALSE;
}

gboolean send_data(int i, char *message)
{
	int result;
	if (i == 1) {
		if (priv_agent1.socket) {
			result = sap_socket_send_data(priv_agent1.socket, MULTI_ACCESSORY1_CHANNELID, strlen(message), message);
		}
	} else if (i == 2) {
		if (priv_agent2.socket) {
			result = sap_socket_send_data(priv_agent2.socket, MULTI_ACCESSORY2_CHANNELID, strlen(message), message);
		}
	}
	return TRUE;

}

static void on_service_connection_requested(sap_peer_agent_h peer_agent,
					    sap_socket_h socket,
					    sap_service_connection_result_e result,
					    void *user_data)
{
	dlog_print(DLOG_DEBUG, TAG, "%s", __func__);
	struct priv *priv = NULL;
	priv = (struct priv *)user_data;
	if (priv->agent_no == 1) {
		priv_agent1.peer_agent = peer_agent;
		priv_agent1.socket = socket;
		update_ui("Connection Established for Peer 1");
	} else if (priv->agent_no == 2) {
		priv_agent2.peer_agent = peer_agent;
		priv_agent2.socket = socket;
		update_ui("Connection Established for Peer 2");
	}
	sap_peer_agent_set_service_connection_terminated_cb
		(peer_agent, on_service_connection_terminated, priv);

	sap_socket_set_data_received_cb(socket, on_data_recieved, priv);

	sap_peer_agent_accept_service_connection(peer_agent);


}

static void on_agent_initialized(sap_agent_h agent,
				 sap_agent_initialized_result_e result,
				 void *user_data)
{
	struct priv *priv = NULL;
	priv = (struct priv *)user_data;

	switch (result) {
	case SAP_AGENT_INITIALIZED_RESULT_SUCCESS:
		dlog_print(DLOG_INFO, TAG, "agents are initialized %d", priv->agent_no);

		sap_agent_set_service_connection_requested_cb(agent,
							      on_service_connection_requested,
							      priv);
		if (priv->agent_no == 1) {
			priv_agent1.agent = agent;
			g_idle_add(_find_peer_agent, &priv_agent1);
		} else if (priv->agent_no == 2) {
			priv_agent2.agent = agent;
			g_idle_add(_find_peer_agent, &priv_agent2);
		}

		break;

	case SAP_AGENT_INITIALIZED_RESULT_DUPLICATED:
		dlog_print(DLOG_ERROR, TAG, "duplicate registration");

		break;

	case SAP_AGENT_INITIALIZED_RESULT_INVALID_ARGUMENTS:
		dlog_print(DLOG_ERROR, TAG, "invalid arguments");

		break;

	case SAP_AGENT_INITIALIZED_RESULT_INTERNAL_ERROR:
		dlog_print(DLOG_ERROR, TAG, "internal sap error");

		break;

	default:
		dlog_print(DLOG_ERROR, TAG, "unknown status (%d)", result);

		break;
	}

}

static void on_device_status_changed(sap_device_status_e status, sap_transport_type_e transport_type,
				     void *user_data)
{
	dlog_print(DLOG_DEBUG, TAG, "%s", __func__);

	switch (transport_type) {
	case SAP_TRANSPORT_TYPE_BT:
		dlog_print(DLOG_DEBUG, TAG, "connectivity type(%d): bt", transport_type);
		break;

	case SAP_TRANSPORT_TYPE_BLE:
		dlog_print(DLOG_DEBUG, TAG, "connectivity type(%d): ble", transport_type);
		break;

	case SAP_TRANSPORT_TYPE_TCP:
		dlog_print(DLOG_DEBUG, TAG, "connectivity type(%d): tcp/ip", transport_type);
		break;

	case SAP_TRANSPORT_TYPE_USB:
		dlog_print(DLOG_DEBUG, TAG, "connectivity type(%d): usb", transport_type);
		break;

	case SAP_TRANSPORT_TYPE_MOBILE:
		dlog_print(DLOG_DEBUG, TAG, "connectivity type(%d): mobile", transport_type);
		break;

	default:
		dlog_print(DLOG_ERROR, TAG, "unknown connectivity type (%d)", transport_type);
		break;
	}

	switch (status) {
	case SAP_DEVICE_STATUS_DETACHED:
		break;

	case SAP_DEVICE_STATUS_ATTACHED:
		break;

	default:
		dlog_print(DLOG_ERROR, TAG, "unknown status (%d)", status);
		break;
	}

}

gboolean agent_initialize()
{
	int result_agent1 = 0, result_agent2 = 0;

	do {
		result_agent1 = sap_agent_initialize(priv_agent1.agent, MULTI_ACCESSORY1_ASPID, SAP_AGENT_ROLE_PROVIDER,
						     on_agent_initialized, &priv_agent1);

		result_agent2 = sap_agent_initialize(priv_agent2.agent, MULTI_ACCESSORY2_ASPID, SAP_AGENT_ROLE_PROVIDER,
						     on_agent_initialized, &priv_agent2);
		dlog_print(DLOG_INFO, TAG, "SAP >>> getRegisteredServiceAgent() >>> %d", result_agent1);
	} while (result_agent1 != SAP_RESULT_SUCCESS || result_agent2 != SAP_RESULT_SUCCESS);

	return TRUE;
}

void initialize_sap()
{
	sap_agent_h agent1 = NULL;
	sap_agent_h agent2 = NULL;

	sap_agent_create(&agent1);
	sap_agent_create(&agent2);

	if (agent1 == NULL)
		dlog_print(DLOG_ERROR, TAG, "ERROR in creating agent");
	else
		dlog_print(DLOG_INFO, TAG, "SUCCESSFULLY create sap agent");

	if (agent2 == NULL)
		dlog_print(DLOG_ERROR, TAG, "ERROR in creating agent");
	else
		dlog_print(DLOG_INFO, TAG, "SUCCESSFULLY create sap agent");

	priv_agent1.agent = agent1;
	priv_agent2.agent = agent2;

	priv_agent1.agent_no = 1;
	priv_agent2.agent_no = 2;

	sap_set_device_status_changed_cb(on_device_status_changed, NULL);
	agent_initialize();
}
